﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Tao.Lua;
using SdlDotNet.Graphics;
using SdlDotNet.Input;
using SdlDotNet.Core;
using Tao.OpenGl;
using System.Drawing;

namespace SPS2
{
    public class SPS2
    {
        public const string SYSTEM_CONFIG_PATH = "system.lua";

        /// <summary>
        /// 垂直方向のアラインメント
        /// </summary>
        public enum VERTICAL_ALIGN
        {
            TOP,
            CENTER,
            BOTTOM,
        }
        /// <summary>
        /// 水平方向のアラインメント
        /// </summary>
        public enum HORIZONTAL_ALIGN
        {
            LEFT,
            CENTER,
            RIGHT,
        }



        public class Flip_Driver
        {
        }


        public Log Log
        {
            get
            {
                return m_Log;
            }
        }

        private SPS2()
        {
            m_Log = new Log();
            Initialize();

            context = new Dictionary<IntPtr, RenderContext>();

            display = new Display();
            display.Width = this.Width;
            display.Height = this.Height;

        }
        public void Load(string path)
        {
            IntPtr L = Lua.luaL_newstate();
            Lua.luaL_openlibs(L);
            LuaFunction.Register(L);

            RenderContext c = new RenderContext(L, path, false);
            RenderContext.Add(L, c);


            Presentation presentation = c.Presentation;

            presentation.ParseAll(c);
            display.presentation = presentation;

            display.scene = new Scene();
            display.pageno = 0;

        }

        private Log m_Log;

        private Display display;

        private static SPS2 m_Instance;

        public static SPS2 Instance
        {
            get
            {
                if (m_Instance == null)
                {
                    m_Instance = new SPS2();
                }
                return m_Instance;
            }
        }
        Dictionary<IntPtr, RenderContext> context;
        public Dictionary<IntPtr, RenderContext> RenderContext
        {
            get
            {
                return context;
            }
        }
        public Display Display
        {
            get { return display; }
        }



        #region Fields

        //Width of screen
        int width = 1024;
        //Height of screen
        int height = 768;
        // Bits per pixel of screen
        int bpp = 16;
        // Surface to render on
        Surface screen;

        /// <summary>
        /// Width of window
        /// </summary>
        protected int Width
        {
            get
            {
                return width;
            }
        }

        /// <summary>
        /// Height of window
        /// </summary>
        protected int Height
        {
            get
            {
                return height;
            }
        }

        /// <summary>
        /// Bits per pixel of surface
        /// </summary>
        protected int BitsPerPixel
        {
            get
            {
                return this.bpp;
            }
        }

        /// <summary>
        /// Lesson title
        /// </summary>
        public static string Title
        {
            get
            {
                return "SPS2 - ";
            }
        }

        #endregion Fields


        #region Lesson Setup
        /// <summary>
        /// Initializes methods common to all NeHe lessons
        /// </summary>
        protected void Initialize()
        {
            // Sets keyboard events
            Events.KeyboardDown += new EventHandler<KeyboardEventArgs>(this.KeyDown);
            // Sets the ticker to update OpenGL Context
            Events.Tick += new EventHandler<TickEventArgs>(this.Tick);
            Events.Quit += new EventHandler<QuitEventArgs>(this.Quit);

            // Set the Frames per second.
            Events.Fps = 60;
            // Sets Window icon and title
            this.WindowAttributes();
            // Creates SDL.NET Surface to hold an OpenGL scene
            screen = Video.SetVideoMode(width, height, true, true, true);
        }

        /// <summary>
        /// Sets Window icon and caption
        /// </summary>
        protected void WindowAttributes()
        {
            Video.WindowIcon();
            Video.WindowCaption = Title;
        }

        /// <summary>
        /// Resizes window
        /// </summary>
        public virtual void Reshape()
        {
            this.Reshape(100.0F);
        }

        /// <summary>
        /// Resizes window
        /// </summary>
        /// <param name="distance"></param>
        public virtual void Reshape(float distance)
        {
            // Reset The Current Viewport
            Gl.glViewport(0, 0, width, height);
            // Select The Projection Matrix
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            // Reset The Projection Matrix
            Gl.glLoadIdentity();
            // Calculate The Aspect Ratio Of The Window
            Glu.gluPerspective(45.0F, (width / (float)height), 0.1F, distance);
            // Select The Modelview Matrix
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            // Reset The Modelview Matrix
            Gl.glLoadIdentity();
        }

        /// <summary>
        /// Initializes the OpenGL system
        /// </summary>
        public virtual void InitGL()
        {
            // Enable Smooth Shading
            Gl.glShadeModel(Gl.GL_SMOOTH);
            // Black Background
            Gl.glClearColor(0.0F, 0.0F, 0.0F, 0.5F);
            // Depth Buffer Setup
            Gl.glClearDepth(1.0F);
            // Enables Depth Testing
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            // The Type Of Depth Testing To Do
            Gl.glDepthFunc(Gl.GL_LEQUAL);
            // Really Nice Perspective Calculations
            Gl.glHint(Gl.GL_PERSPECTIVE_CORRECTION_HINT, Gl.GL_NICEST);
        }

        #endregion Lesson Setup

        #region void DrawGLScene

        /// <summary>
        /// Renders the scene
        /// </summary>
        protected virtual void DrawGLScene()
        {
            //Gl.glClearColor(1.0f, 0.0f, 0.0f, 0.4f);
            Color bgcolor = display.scene.page.setting.backcolor;
            Gl.glClearColor((float)bgcolor.R / 255.0f, (float)bgcolor.G / 255.0f, (float)bgcolor.B / 255.0f , (float)bgcolor.A / 255.0f);

            // Clear Screen And Depth Buffer
            Gl.glClear((Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT));
            // Reset The Current Modelview Matrix
            Gl.glLoadIdentity();

            Enter2DMode();
            Gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

            switch (display.state)
            {
                case 0:
                    display.scene.Draw(display);
                    break;
                case 1:
                    display.flip_param.tick_current = DateTime.Now.Ticks;
                    display.Flip();
                    break;
            }

            Leave2DMode();

        }
        protected void Enter2DMode()
        {
            Gl.glPushAttrib(Gl.GL_ENABLE_BIT);
            Gl.glDisable(Gl.GL_DEPTH_TEST);
            Gl.glDisable(Gl.GL_CULL_FACE);
            Gl.glEnable(Gl.GL_TEXTURE_2D);
            Gl.glEnable(Gl.GL_BLEND);
            Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
            Gl.glViewport(0, 0, Width, Height);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glPushMatrix();
            Gl.glLoadIdentity();

            Gl.glOrtho(-Width / 2.0f, Width / 2.0f, Height / 2.0f, -Height / 2.0f, 0.0f, 10.0f);

            //Gl.glFrustum(-(float)Width / 2.0f, (float)Width / 2.0f, (float)Height / 2.0f, -(float)Height / 2.0f, 1.0f, 10.0f);
            //Gl.glTranslatef(0.0f, 0.0f, -1.0f); 

            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glPushMatrix();
            Gl.glLoadIdentity();
            Gl.glTexEnvf(Gl.GL_TEXTURE_ENV, Gl.GL_TEXTURE_ENV_MODE, Gl.GL_MODULATE);
        }
        protected void Leave2DMode()
        {
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glPopMatrix();
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glPopMatrix();
            Gl.glPopAttrib();
        }

        private void Next()
        {


            if (display.scene.pauselevel == display.scene.page.pauselevel_max)
            {
                display.flip_param = new Flip.Parameter();
                display.flip_param.display = display;
                display.flip_param.scene_before = new Scene(display.scene);

                // next page
                if (display.pageno == display.presentation.pageno_max)
                {
                    display.pageno = 0;
                }
                else
                {
                    display.pageno++;
                }
                display.flip_param.scene_after = display.scene;
                display.flip_param.tick_begin = DateTime.Now.Ticks;
                display.flip_param.tick_current = DateTime.Now.Ticks;
                display.state = 1;

            }
            else
            {
                display.scene.pauselevel++;
            }
            
        }
        private void Prev()
        {
            if (display.scene.pauselevel == 0)
            {
                // previous page
                if (display.pageno == 0)
                {
                    display.pageno = display.presentation.pageno_max;
                }
                else
                {
                    display.pageno--;
                }
            }
            else
            {
                display.scene.pauselevel--;
            }

        }
        private void SpaceKey()
        {
            switch (display.state)
            {
                case 0:
                    Next();
                    break;
                case 1:
                    display.state = 0;
                    break;
            }
        }
        private void BackSpaceKey()
        {
            switch (display.state)
            {
                case 0:
                    Prev();
                    break;
                case 1:
                    display.state = 0;
                    break;
            }
        }
        #endregion void DrawGLScene

        #region Event Handlers

        private void KeyDown(object sender, KeyboardEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Escape:
                    // Will stop the app loop
                    Events.QuitApplication();
                    break;
                case Key.Space:
                    SpaceKey();
                    break;
                case Key.Backspace:
                    BackSpaceKey();
                    break;
                case Key.F1:
                    // Toggle fullscreen
                    if ((screen.FullScreen))
                    {
                        screen = Video.SetVideoMode(width, height, true, true, true);
                        this.WindowAttributes();
                    }
                    else
                    {
                        screen = Video.SetVideoMode(width, height, true, true);
                    }
                    Reshape();
                    break;
            }
        }

        private void Tick(object sender, TickEventArgs e)
        {
            DrawGLScene();
            Video.GLSwapBuffers();
        }

        private void Quit(object sender, QuitEventArgs e)
        {
            Events.QuitApplication();
        }

        #endregion Event Handlers



    }
}
